;;; url-sdp.el --- File retrieval code
;; Author: kuriya@kuis.kyoto-u.ac.jp, fujikawa@kuis.kyoto-u.ac.jp
;;
;; Ex: sdp://london-station.bbcc.com:ttl=16/sdp+test#m=audio+10000+RTP-AVP+0
;; See ftp://ftp.ietf.org/internet-drafts/draft-fujikawa-sdp-url-01.txt

(require 'url-vars)
(require 'url-parse)
(defmacro sdp-type (sdpobj)
  (` (aref (, sdpobj) 0)))

(defmacro sdp-multiaddr (sdpobj)
  (` (aref (, sdpobj) 1)))

(defmacro sdp-ttl (sdpobj)
  (` (aref (, sdpobj) 2)))

(defmacro sdp-noa (sdpobj)
  (` (aref (, sdpobj) 3)))

(defmacro sdp-session (sdpobj)
  (` (aref (, sdpobj) 4)))

(defmacro sdp-info (sdpobj)
  (` (aref (, sdpobj) 5)))

(defmacro sdp-maddr-v (sdpobj)
kuriya@kuis.kyoto-u.ac.jp  (` (aref (, sdpobj) 6)))

(defmacro sdp-port-v (sdpobj)
  (` (aref (, sdpobj) 7)))

(defmacro sdp-maddr-a (sdpobj)
  (` (aref (, sdpobj) 8)))

(defmacro sdp-port-a (sdpobj)
  (` (aref (, sdpobj) 9)))


(defun url-sdp (url)
  (let* ((sdpobj (url-generic-parse-sdp url))
	 (v-addr-port (concat (sdp-maddr-v sdpobj) "/" (sdp-port-v sdpobj)))
	 (a-addr-port (concat (sdp-maddr-a sdpobj) "/" (sdp-port-a sdpobj))))
    (get-buffer-create "foo")
    (if (not (null (sdp-maddr-v sdpobj)))
	(start-process "my-process" "foo" "vic" "-C" (sdp-session sdpobj) v-addr-port ))
    (if (not (null (sdp-maddr-a sdpobj)))
	(start-process "my-process" "foo" "vat" "-C" (sdp-session sdpobj) a-addr-port ))
    (message (sdp-info sdpobj))))


(defun url-generic-parse-sdp (url)
;  "Return a vector of the parts of SDP-URL.
;Format is:
;[prot maddr ttl noa session info maddr-v port-v maddr-a port-a]"
  (save-excursion
    (set-buffer (get-buffer-create " *sdpparse*"))
    (set-syntax-table url-mailserver-syntax-table)
    (let*  ((save-pos nil)
	    (prot nil)
	    (maddr-ttl-noa nil)
	    (maddr nil)
	    (ttl-noa nil)
	    (ttl nil)
	    (noa nil)
	    (session nil)
	    (info nil)
	    (maddr-v nil)
	    (port-v nil)
	    (maddr-a nil)
	    (port-a nil)
	    (inhibit-read-only t))
      (erase-buffer)
      (insert url)
      (subst-char-in-region (point-min) (point-max) ?\+ ?\ )
      (goto-char (point-min))
      (setq save-pos (point))
      (if (not (looking-at "//"))
	  (progn
	    (skip-chars-forward "a-zA-Z+.\\-")
	    (downcase-region save-pos (point))
	    (setq prot (buffer-substring save-pos (point)))
	    (skip-chars-forward ":")
	    (setq save-pos (point))))

      ;; We are doing a fully specified URL, with hostname and all
      (if (looking-at "//")
	  (progn
	    (setq full t)
	    (forward-char 2)
	    (setq save-pos (point))
	    (skip-chars-forward "^/")
	    (setq maddr-ttl-noa (buffer-substring save-pos (point)))
	    (if (string-match "^\\([^:]+\\):" maddr-ttl-noa)
		(setq maddr (url-match maddr-ttl-noa 1)
		      ttl-noa (substring maddr-ttl-noa (match-end 0) nil))
	      (setq maddr maddr-ttl-noa))
	    (if (and ttl-noa (string-match "^ttl=\\([^:]+\\):" ttl-noa))
		(progn
		  (setq ttl (url-match ttl-noa 1)
			noa (substring ttl-noa (match-end 0) nil))
		  (if (string-match "noa=" ttl-noa)
		      (setq noa (substring ttl-noa (match-end 0) nil))))
	      (if (and ttl-noa (string-match "^ttl=\\([^:]+\\)" ttl-noa))
		    (setq ttl (url-match ttl-noa 1)))
	      (if (and ttl-noa (string-match "^noa=\\([^:]+\\)" ttl-noa))
		  (setq noa (url-match ttl-noa 1))))
	    (setq save-pos (point))))
      (if (looking-at "/")
	  (progn
	    (forward-char 1)
	    (setq save-pos (point))
	    (skip-chars-forward "^#")
	    (setq session (buffer-substring save-pos (point)))
	    (setq save-pos (point))))
      (if (looking-at "#")
	  (let ((media nil)
		(attributes nil))
	    (forward-char 1)
	    (setq save-pos (point))
	    (skip-chars-forward "^&")
	    (setq attributes (buffer-substring save-pos (point)))
	    (while (not (null attributes))
	      (if (string-match "^s=\\([^&]+\\)" attributes)
		  (setq session (url-match attributes 1)))
	      (if (string-match "^i=\\([^&]+\\)" attributes)
		  (setq info (url-match attributes 1)))
	      (if (string-match "^m=\\([^&]+\\)" attributes)
		  (progn
		    (setq attributes (url-match attributes 1))
		    (if (string-match "^video \\([0-9]+\\)" attributes)
			(progn
			  (setq media "video")
			  (setq maddr-v maddr)
			  (setq port-v (url-match attributes 1))))
		    (if (string-match "^audio \\([0-9]+\\)" attributes)
			(progn
			  (setq media "audio")
			  (setq maddr-a maddr)
			  (setq port-a (url-match attributes 1))))
		    )
		)
	      (if (looking-at "&")
		  (progn
		    (forward-char 1)
		    (setq save-pos (point))
		    (skip-chars-forward "^&")
		    (setq attributes (buffer-substring save-pos (point))))
		(setq attributes nil))
	      )
	    )
	)

      (vector prot maddr ttl noa session info maddr-v port-v maddr-a port-a))))




